#include "examplewindow.h"
#include <boost/algorithm/string/replace.hpp>
#include <iostream>

ExampleWindow::ExampleWindow() : m_button_buffer1("جستجو"),
                                 m_label("نویسه شمار"),
                                 m_labelTwo("نویسهٔ مورد جستجو:"),
                                 m_labelThree("نتیجه جستجو"),
                                 full_character(0),
                                 custom_character(0)
{
    style();
    set_title("");
    set_default_size(500, 50);
    set_default_direction(Gtk::TEXT_DIR_RTL);
    set_position(Gtk::WindowPosition::WIN_POS_CENTER);

    m_button_buffer1.signal_clicked().connect(sigc::mem_fun(*this, &ExampleWindow::on_button_buffer1));

    m_vBoxOne.set_margin_right(10);
    m_vBoxOne.set_margin_left(10);
    m_hBoxOne.set_margin_bottom(10);
    //m_hBoxOne.set_margin_top(20);

    m_vBoxOne.set_spacing(10);
    m_hBoxOne.set_spacing(10);
    m_vBoxTwo.set_spacing(10);
    m_hBoxTwo.set_spacing(10);

    m_button_buffer1.set_size_request(200, 20);
    m_labelThree.set_size_request(300, 200);

    m_entry.set_width_chars(5);
    m_entry.set_max_length(1);
    m_entry.set_alignment(Gtk::ALIGN_CENTER);

    auto css = Gtk::CssProvider::create();
    auto win = get_style_context();
    auto screen = Gdk::Screen::get_default();

    try
    {
        if ( ! css->load_from_data(strStyle))
        {
            std::cout << "ERROR!";
        }
    }
    catch( Gtk::CssProviderError &e )
    {
        std::cerr << "WARNING: theme.css not load.";
    }

    //m_label.set_valign(Gtk::ALIGN_START);

    m_scrolledWindow.set_shadow_type(Gtk::SHADOW_ETCHED_IN);

    win->add_provider_for_screen(screen, css, GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);

    m_scrolledWindow.add(m_textView);
    m_scrolledWindow.set_size_request(200, 300);
    //m_scrolledWindow.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS);
    m_textView.set_wrap_mode(Gtk::WRAP_WORD);

    m_vBoxOne.add(m_label);
    m_vBoxOne.add(m_hBoxOne);

    m_hBoxOne.add(m_vBoxTwo);
    m_hBoxOne.add(m_labelThree);

    m_vBoxTwo.add(m_scrolledWindow);
    m_vBoxTwo.add(m_hBoxTwo);

    m_hBoxTwo.add(m_labelTwo);
    m_hBoxTwo.add(m_entry);
    m_hBoxTwo.add(m_button_buffer1);

    add(m_vBoxOne);

    show_all();
}

ExampleWindow::~ExampleWindow()
{

}

void ExampleWindow::bufferOne()
{
    m_refTextBufferOne = Gtk::TextBuffer::create();
    m_refTextBufferOne = m_textView.get_buffer();
    str_utf8_one = m_refTextBufferOne->get_text();
}

void ExampleWindow::bufferTwo()
{
    m_refEntryBufferOne = Gtk::EntryBuffer::create();
    m_refEntryBufferOne = m_entry.get_buffer();
    str_utf8_two = m_refEntryBufferOne->get_text();
}

std::string ExampleWindow::en_to_per_num(std::string str)
{
    boost::replace_all( str, "0", "۰" );
    boost::replace_all( str, "1", "۱" );
    boost::replace_all( str, "2", "۲" );
    boost::replace_all( str, "3", "۳" );
    boost::replace_all( str, "4", "۴" );
    boost::replace_all( str, "5", "۵" );
    boost::replace_all( str, "6", "۶" );
    boost::replace_all( str, "7", "۷" );
    boost::replace_all( str, "8", "۸" );
    boost::replace_all( str, "9", "۹" );

    return str;
}

void ExampleWindow::on_button_buffer1()
{
    bufferOne();
    bufferTwo();

    str_utf16_one = convert.from_bytes(str_utf8_one);
    str_utf16_two = convert.from_bytes(str_utf8_two);

    if (str_utf16_one.length() == 0)
    {
        m_labelThree.set_text("تعداد نویسه مورد جستجو: ۰\nتعداد کل نویسه‌های متن:  ۰");

        //std::cout << "۰" << std::endl;
    }
    else
    {
        for (int i = 0; i < str_utf16_one.length(); ++i)
        {
            std::string str1 = convert.to_bytes(str_utf16_one.at(i));
            //std::cout << str1 << std::endl;

            ++full_character;

            //std::cout << en_to_per_num(std::to_string(full_character)) << std::endl;

            for (int j = 0; j < str_utf16_two.length(); ++j)
            {
                std::string str2 = convert.to_bytes(str_utf16_two.at(j));
                //std::cout << " " << str2 << std::endl;

                if (str1 == str2)
                {
                    ++custom_character;
                }
            }
        }

        std::string tmp = "تعداد نویسه مورد جستجو:  " +
                          en_to_per_num(std::to_string(custom_character)) +
                          "\n" +
                          "تعداد کل نویسه‌های متن:   " +
                          en_to_per_num(std::to_string(full_character));

        m_labelThree.set_text(tmp);

        custom_character = 0;
        full_character = 0;
    }
}

void ExampleWindow::style()
{
    strStyle =
    "window {"
      "background-color: #444444;"
    "}"

    "label {"
      "font: XB Niloofar 16;"
      "color: white;"
    "}"

    "entry {"
      "background-color: gray;"
      "background-image: none;"
      "color: white;"
      "border-image: none;"
      "border-radius: 50px;"
      "border-color: #444444;"
    "}"

    "button {"
      "background-color: #cc0000;"
      "background-image: none;"
      "border-image: none;"
      "border-radius: 50px;"
    "}"

    "button:hover:active {"
      "background-color: #b30000;"
    "}"

    "textview {"
      "font: XB Niloofar 16;"
    "}"

    "scrolledwindow {"
      "border-width: 10px;"
      "border-radius: 10px;"
      "border-color: white;"
    "}";
}
